'' =================================================================================================
'   File......  TrillionYearClock.spin    
'   Purpose...  Trillion Year Clock Telnet RGB Led Timer Eeprom Demo
'   Author....  Clock Loop @ parallax forums 
'   Started...  2016...
'   Updated...  2018... See you in a trillion years.
'' =================================================================================================

CON

  _CLKMODE = XTAL1 + PLL16X
  _XINFREQ = 5_000_000
          
CON

  Blue           = 15      
  Green          = 16
  Red            = 17

  Tx             = 30   'serial 
  Rx             = 31   'serial 

  'EepromScl      = 28  'eeprom. 
  'EepromSda      = 29  'eeprom.  

  
  'Assuming that a 80 MHz core clock is being used, 800 cycles is about 10us.
  'in_pasm(1 + cnt)             'Wait for 12.5 ns   -80 Mhz 
  'in_pasm(8 + cnt)             'Wait for 100 ns    -10 Mhz  
  'in_pasm(80 + cnt)            'Wait for 1us       -1 Mhz      
  'waitcnt(800 + cnt)           'Wait for 10us      -100 Khz
  'waitcnt(8_000 + cnt)         'Wait for 100 us    -10 Khz  
  'waitcnt(80_000 + cnt)        'Wait for 1ms       -1 Khz
  'waitcnt(800_000 + cnt)       'Wait for 10 ms     -100 Hz
  'waitcnt(8_000_000 + cnt)     'Wait for 100 ms    -10 Hz human reaction speed
  'waitcnt(80_000_000 + cnt)    'Wait for 1000 ms   -1 Hz
{{   
    Planck
    In 2015, the Planck Collaboration estimated the age of the universe to be 13.813±0.038 billion years, 
    slightly higher but within the uncertainties of the earlier number derived from the WMAP data. 
    By combining the Planck data with external data, the best combined estimate of the age of the universe is:
     (13.799±0.021)×10^9 years old.    13,799,792,018.342.15.51.42 years old.
     
     To sync to your local time: Change these values in the DAT section.
     
  uSeconds       == 0     'Your local seconds, 0-59       
  uMinute        == 0     'Your local minutes, 0-59       
  uHour          == 0     'Your local hour, 24 hour format, 0-23      
  uDays          == 0     'Your local day, 365 day format, 0-364    
  uYears         == 8     'Your Year (for year 2018, its 8)
  uDecade        == 1     'Your Decade (for year 2018, its 1)
  uCentury       == 0     'Your century (for year 2018, its 0)
  uMillennium    == 792   'Your thousandth year (for year 2018, its any 3 digit 'age of universe' fudge number that ENDS in 2)
  uMillionYears  == 799   'Calculated from age of universe. 799(13.799±0.021)×10^9 years old. 
  uBillionYears  == 13    'Calculated from age of universe. 13(13.799±0.021)×10^9 years old.
  uTrillionYears == 0     'Calculated from age of universe. 0(13.799±0.021)×10^9 years old.

}}  

DAT

  uSeconds              long    0
  uMinute               long    40
  uHour                 long    6
  uDays                 long    344
  uYears                long    8
  uDecade               long    1
  uCentury              long    0
  uMillennium           long    792
  uMillionYears         long    799
  uBillionYears         long    13,0,0,0,0,0,0,0,0,0
  uTrillionYears        long    0,0,0,0,0,0,0,0,0,0
  uInfinite              long    0,0,0,0,0,0,0,0,0,0
  
VAR

  long  cogon[8], cog[8], pile[800]   ' cog management
  long  Time
  
OBJ

  Term          : "FullDuplexSerialPlus"  'Data out to Putty On ParallaxESP Port 23
  eeram         : "jm_24xx256" 'Routines to read from and write to eerom.
  

PUB  Start

  Stop     'stop all cogs if running  
 
  term.start(Rx, Tx, %0000, 921_600)                                    '1cog start terminal

  waitcnt(8_000_000 + cnt)                                              'wait 1/10 second 

  term.str(string(27))                                                  'Set command mode for telnet          
  term.str(string("[2J"))                                               'Clear telnet screen
        
  eeram.start(%000)                                                     '1cog starts but exits.  

  waitcnt(8_000_000 + cnt)                                              'wait 1/10 second 
   
  cogon[0] := ((cog[0] := cognew(CoreLoop, @pile[0]) + 1) > 0)         '1cog    Timing system, blinks leds, records on-time(since poweron), and total run-time(since last programming) to eeprom. 
  
  cogon[1] := ((cog[1] := cognew(Display, @pile[100]) + 1) > 0)         '1cog    Loop to time how often to paint the screen with data.
  
  'cogon[2] := ((cog[2] := cognew(Nothing0, @pile[200]) + 1) > 0)           
  
  'cogon[3] := ((cog[3] := cognew(Nothing1, @pile[300]) + 1) > 0)         
  
  'cogon[4] := ((cog[4] := cognew(Nothing2, @pile[400]) + 1) > 0)         

  'cogon[5] := ((cog[5] := cognew(Nothing3, @pile[500]) + 1) > 0)       
  
  'cogon[6] := ((cog[6] := cognew(Nothing4, @pile[600]) + 1) > 0)    

 TenthCounter 'The 8th cog goes here, remember its also the main cog that booted the prop.

Pub TenthCounter | tempo

  Repeat
     
    waitcnt(7_999_032 + cnt)     'wait 1/10 second                
    Trigger := 1  
                          '
Pub Display | old    
  
  Repeat
     
     If TenthTimer <> old

         old := TenthTimer         
         
         If TenthTimer == 0
             PaintScreen
             DotOn
             
         ElseIf TenthTimer == 5
             PaintScreen
             DotOff
  
Pub CoreLoop

  outa[Red] := 1
  outa[Green] := 1
  outa[Blue] := 1

  dira[Red] := 1   
  dira[Green] := 1
  dira[Blue] := 1

  Ledz := 1
                             '
 Repeat

  If Trigger == 1
    Trigger := 0  
    
      If TenthTimer == 9        
        TenthTimer := 0
        uGoSecond  
        GoSecond  
      Else
        TenthTimer++          

    If Ledz == 1
        outa[Red] := 0 
     
    If Ledz == 2
        outa[Red] := 1
     
    If Ledz == 3
        outa[Blue] := 0
        
    If Ledz == 4
        outa[Blue] := 1
     
    If Ledz == 5 
        outa[Green] := 0
        
    If Ledz == 6                         
        outa[Green] := 1
     
    Ledz++
    If Ledz > 6
        Ledz := 1

Pub uGoSecond
  If uSeconds > 58    
    uSeconds := 0
    'eeram.wr_long(@uSeconds, uSeconds)   Do not store constantly changing values to eeprom, it might get damaged.
        uGoMinute
  Else             
    uSeconds := uSeconds + 1
    'eeram.wr_long(@uSeconds, uSeconds)   Do not store constantly changing values to eeprom, it might get damaged.                                         
               
Pub uGoMinute             
  If uMinute > 58
    uMinute := 0 
    'eeram.wr_long(@uMinute, uMinute)  Do not store constantly changing values to eeprom, it might get damaged.     
        uGoHour
  Else
    uMinute := uMinute + 1
    'eeram.wr_long(@uMinute, uMinute)  Do not store constantly changing values to eeprom, it might get damaged.
       
Pub uGoHour   
  If uHour > 22
    uHour := 0
    eeram.wr_long(@uHour, uHour)      
        uGoDay     
  Else
    uHour := uHour + 1                 
    eeram.wr_long(@uHour, uHour)            
                        
Pub uGoDay            
  If uDays > 363
    uDays := 0
    eeram.wr_long(@uDays, uDays)      
        uGoYear      
  Else
    uDays := uDays + 1
    eeram.wr_long(@uDays, uDays)       
                    
Pub uGoYear                    
  If uYears > 8
    uYears := 0
    eeram.wr_long(@uYears, uYears)      
        uGoDecade       
  Else
    uYears := uYears + 1
    eeram.wr_long(@uYears, uYears)                                          
                        
Pub uGoDecade                        
  If uDecade > 8
    uDecade := 0
    eeram.wr_long(@uDecade, uDecade)                      
        uGoCentury    
  Else
    uDecade := uDecade + 1
    eeram.wr_long(@uDecade, uDecade)      
                     
Pub uGoCentury                    
  If uCentury > 8
    uCentury := 0
    eeram.wr_long(@uCentury, uCentury)                      
        uGoMillennium   
  Else  
    uCentury := uCentury + 1
    eeram.wr_long(@uCentury, uCentury)     
                             
Pub uGoMillennium                                                             
  If uMillennium > 998
    uMillennium := 0
    eeram.wr_long(@uMillennium, uMillennium)                      
        uGoMillion                                                                                                
  Else
    uMillennium := uMillennium + 1
    eeram.wr_long(@uMillennium, uMillennium)      

Pub uGoMillion   
  If uMillionYears > 998
    uMillionYears := 0
    eeram.wr_long(@uMillionYears, uMillionYears)                                                            
        uGoBillion                                                                                                                                                                                                                                                                                              
  Else
    uMillionYears := uMillionYears + 1
    eeram.wr_long(@uMillionYears, uMillionYears)      
                                   
Pub uGoBillion
  If uBillionYears > 998
    uBillionYears := 0
    eeram.wr_long(@uBillionYears, uBillionYears)                                                           
        uGoTrillion
  Else
    uBillionYears := uBillionYears + 1
    eeram.wr_long(@uBillionYears, uBillionYears)         
    
Pub uGoTrillion
    If uTrillionYears < 0
      uTrillionYears := 0
    
    uTrillionYears := uTrillionYears + 1 'This will overflow at 2,147,483,647, so you better be there to write it down on a piece of paper.
    eeram.wr_long(@uTrillionYears, uTrillionYears)   'This will overflow at 2,147,483,647, so you better be there to write it down on a piece of paper.  
    TrillionYears := TrillionYears + 1 'This will overflow at 2,147,483,647, so you better be there to write it down on a piece of paper.            



Pub GoSecond
  If Seconds > 58    
    Seconds := 0
        GoMinute
  Else             
    Seconds := Seconds + 1                                              
               
Pub GoMinute             
  If Minute > 58
    Minute := 0
        GoHour
  Else
    Minute := Minute + 1 
       
Pub GoHour   
  If Hour > 22
    Hour := 0     
        GoDay     
  Else
    Hour := Hour + 1                 
                   
Pub GoDay            
  If Days > 363
    Days := 0
        GoYear      
  Else
    Days := Days + 1       
                    
Pub GoYear                    
  If Years > 8
    Years := 0
        GoDecade       
  Else
    Years := Years + 1                                        
                        
Pub GoDecade                        
  If Decade > 8                  
    Decade := 0
        GoCentury    
  Else  
    Decade := Decade + 1  
                     
Pub GoCentury                    
  If Century > 8                   
    Century := 0 
        GoMillennium   
  Else   
    Century := Century + 1
                             
Pub GoMillennium                                                             
  If Millennium > 998                   
    Millennium := 0 
        GoMillion                                                                                                
  Else
    Millennium := Millennium + 1    

Pub GoMillion   
  If MillionYears > 998                    
    MillionYears := 0                                        
        GoBillion                                                                                                                                                                                                                                                                                              
  Else
    MillionYears := MillionYears + 1     
                                   
Pub GoBillion
  If BillionYears > 998                    
    BillionYears := 0                                      
        GoTrillion
  Else    
    BillionYears := BillionYears + 1    
    
Pub GoTrillion
    If TrillionYears < 0
      TrillionYears := 0
    TrillionYears := TrillionYears + 1 'This will overflow at 2,147,483,647, so you better be there to write it down on a piece of paper.            

      
Pub  PaintScreen 
     
      If Seconds == 0        
        If TenthTimer == 0           
          term.str(string(27))  'Set command mode          
          term.str(string("[2J")) 'clear screen      
      
      term.str(string(27))    'Set command mode 
      term.str(string("[1;1f"))  'Set Position
            
      term.str(string(27))  'Set command mode  
      term.str(string("[2;3f")) 'position

      term.str(string(27))   'Set command mode  
      term.str(string("[35m")) 'Purple color text

      term.str(string(27))   'Set command mode
      term.str(string("[1m")) 'Turn bold mode on 

      term.str(string(27))   'Set command mode 
      term.str(string("#3"))  'Double-height letters, top half 
      term.str(string("Trillion Year Clock"))

      term.str(string(27))    'Set command mode 
      term.str(string("[1;1f"))  'Set Position

      term.str(string(27))  'Set command mode  
      term.str(string("[3;3f")) 'position

      term.str(string(27))   'Set command mode 
      term.str(string("#4"))  'Double-height letters, bottom half 
      term.str(string("Trillion Year Clock")) 
      
      term.str(string(27))  'Set command mode  
      term.str(string("[2;25f")) 'position 

      term.str(string(27))    'Set command mode
      term.str(string("[33m")) 'Yellow color text
                        
      term.str(string(27))   'Set command mode 
      term.str(string("#3"))  'Double-height letters, top half 
      term.str(string("Ticker   : "))
                                     
      If Seconds < 10
        term.dec(0)
      term.dec(Seconds)           
      
      term.str(string(27))    'Set command mode 
      term.str(string("[2;39f"))  'Set Position
        
      term.dec(TenthTimer)
      term.str(string("   "))

      term.str(string(27))  'Set command mode  
      term.str(string("[3;25f")) 'position

      term.str(string(27))   'Set command mode 
      term.str(string("#4"))  'Double-height letters, bottom half 
      term.str(string("Ticker   : ")) 
  
      If Seconds < 10
        term.dec(0)
      term.dec(Seconds)
      
      term.str(string(27))    'Set command mode 
      term.str(string("[3;39f"))  'Set Position
        
      term.dec(TenthTimer)
      term.str(string("   "))
      
      term.str(string(27))    'Set command mode
      term.str(string("[0m")) 'Turn off character attributes
                              
      term.str(string(27))    'Set command mode 
      term.str(string("[1;1f"))  'Set Position     
                             
    '----------------------------------------------
        'term.str(string(27))  'Set command mode          
        'term.str(string("[2J")) 'clear screen
                  
        'term.str(string(27))    'Set command mode
        'term.str(string("[0m")) 'Turn off character attributes
        'term.str(string("[1m")) 'Turn bold mode on
        'term.str(string("[2m")) 'Turn low intensity mode on
        'term.str(string("[4m")) 'Turn underline mode on
        'term.str(string("[5m")) 'Turn blinking mode on
        'term.str(string("[7m")) 'Turn reverse video on    
        'term.str(string("[8m")) 'Turn invisible text mode on
     
        'term.str(string("#3"))  'Double-height letters, top half 
        { 
        Esc#3   Double-height letters, top half         DECDHL
        Esc#4   Double-height letters, bottom half      DECDHL
        Esc#5   Single width, single height letters     DECSWL
        Esc#6   Double width, single height letters     DECDWL
        }

        'term.str(string(27))  'Set command mode 
        'term.str(string("[30m")) 'Black color text
        'term.str(string("[31m")) 'Red color text
        'term.str(string("[32m")) 'Lime color text
        'term.str(string("[33m")) 'Yellow color text
        'term.str(string("[34m")) 'Blue color text  
        'term.str(string("[35m")) 'Purple color text
        'term.str(string("[36m")) 'SkyBlue color text  
        'term.str(string("[37m")) 'White color text
                                     
        term.str(string(27))    'Set command mode 
        term.str(string("[5;5f"))  'Set Position 
                                    '
        term.str(string(27))  'Set command mode 
        term.str(string("[32m")) 'Lime color text
        
        term.str(string("RunTime  : "))  

        term.dec(uTrillionYears)       
        term.str(string(":TrillionYears  "))
                                   
        term.dec(uBillionYears)       
        term.str(string(":BillionYears   "))
        
        term.dec(uMillionYears)       
        term.str(string(":MillionYears   "))
        
        term.dec(uMillennium)       
        term.str(string(":Millennium  "))
        
        term.str(string(27))    'Set command mode 
        term.str(string("[6;16f"))  'Set Position 
        
        term.dec(uCentury)       
        term.str(string(":Century   "))
        
        term.dec(uDecade)       
        term.str(string(":Decade   "))
        
        term.dec(uYears)       
        term.str(string(":Years  "))
        
        term.dec(uDays)       
        term.str(string(":Days   "))
        
        term.dec(uHour)       
        term.str(string(":Hours   "))

        term.dec(uMinute)       
        term.str(string(":Minute   "))

        term.str(string(27))    'Set command mode 
        term.str(string("[1;1f"))  'Set Position

        term.str(string(27))    'Set command mode 
        term.str(string("[8;5f"))  'Set Position 

        term.str(string(27))  'Set command mode 
        term.str(string("[31m")) 'Red color text  
        
        term.str(string("UpTime   : "))

        term.dec(TrillionYears)       
        term.str(string(":TrillionYears  "))
        
        term.dec(BillionYears)       
        term.str(string(":BillionYears   "))
        
        term.dec(MillionYears)       
        term.str(string(":MillionYears   "))
        
        term.dec(Millennium)       
        term.str(string(":Millennium  "))
        
        term.str(string(27))    'Set command mode 
        term.str(string("[9;16f"))  'Set Position 
        
        term.dec(Century)       
        term.str(string(":Century   "))
        
        term.dec(Decade)       
        term.str(string(":Decade   "))

        term.dec(Years)       
        term.str(string(":Years  "))
        
        term.dec(Days)       
        term.str(string(":Days   "))
        
        term.dec(Hour)       
        term.str(string(":Hours   "))

        term.dec(Minute)       
        term.str(string(":Minute   "))   
              

Pub DotOff

    term.str(string(27))    'Set command mode 
    term.str(string("[2;38f"))  'Set Position
                                '
    term.str(string(27))   'Set command mode
    term.str(string("[1m")) 'Turn bold mode on 
                            
    term.str(string(27))   'Set command mode  
    term.str(string("[35m")) 'Purple color text                            
                              
    term.str(string(27))   'Set command mode 
    term.str(string("#3"))  'Double-height letters, top half 
                            
    term.str(string("."))                               
      
    term.str(string(27))    'Set command mode 
    term.str(string("[3;38f"))  'Set Position
                                
    term.str(string(27))   'Set command mode 
    term.str(string("#4"))  'Double-height letters, bottom half                           
       
    term.str(string("."))
    
    term.str(string(27))    'Set command mode 
    term.str(string("[1;1f"))  'Set Position                                                        
                                                                
Pub DotOn

    term.str(string(27))    'Set command mode 
    term.str(string("[2;38f"))  'Set Position
                                '
    term.str(string(27))   'Set command mode
    term.str(string("[1m")) 'Turn bold mode on 
                            
    term.str(string(27))   'Set command mode  
    term.str(string("[37m")) 'White color text                            
                              
    term.str(string(27))   'Set command mode 
    term.str(string("#3"))  'Double-height letters, top half 
                            
    term.str(string("."))                               
      
    term.str(string(27))    'Set command mode 
    term.str(string("[3;38f"))  'Set Position
                                
    term.str(string(27))   'Set command mode 
    term.str(string("#4"))  'Double-height letters, bottom half                           
       
    term.str(string("."))
    
    term.str(string(27))    'Set command mode 
    term.str(string("[1;1f"))  'Set Position                                                                              
                      
PUB Stop

    if cogon[0]                                          ' if object running, mark stopped
        cogstop(cog[0]~ - 1)    

    if cogon[1]                                          ' if object running, mark stopped
        cogstop(cog[1]~ - 1)
    
    if cogon[2]                                          ' if object running, mark stopped
        cogstop(cog[2]~ - 1)

    if cogon[3]                                          ' if object running, mark stopped
        cogstop(cog[3]~ - 1)
 
    if cogon[4]                                          ' if object running, mark stopped
        cogstop(cog[4]~ - 1)

    if cogon[5]                                          ' if object running, mark stopped
        cogstop(cog[5]~ - 1)

    if cogon[6]                                         ' if object running, mark stopped
        cogstop(cog[6]~ - 1)
    
Dat
  Trigger               long    0
  TenthTimer            long    0
  
  Seconds               long    0
  Minute                long    0
  Hour                  long    0
  Days                  long    0
  Years                 long    0
  Decade                long    0 
  Century               long    0
  Millennium            long    0
  MillionYears          long    0
  BillionYears          long    0,0,0,0,0,0,0,0,0,0
  TrillionYears         long    0,0,0,0,0,0,0,0,0,0
  Infinite               long    0,0,0,0,0,0,0,0,0,0
                  
  Ledz                  long    0
  
Dat

{{   
    Planck

    In 2015, the Planck Collaboration estimated the age of the universe to be 13.813±0.038 billion years, 
    slightly higher but within the uncertainties of the earlier number derived from the WMAP data. 
    By combining the Planck data with external data, the best combined estimate of the age of the universe is:
     (13.799±0.021)×10^9 years old.    13,799,000,000.000.00.00.00 years old.
     TrilionYear.BillionYear.MillionYear.Millennium.Century.Decade.Year.Day.Hour.Minute.Second
}}

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

}}
